/* Tobin's nifty C-to-HTML syntax hilighter
    Copyright (c) 2000 by Tobin Fricke <tobin@splorg.org> 

   Note than the real meat of this program actually resides in the
   file lex.yy.c, which is in turn generated by GNU flex from the
   source file prettyc.flex.  This component (prettyc.c) is just
   a driver loop that calls yylex() defined in lex.yy.c.

*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "prettyc.h"

/* These symbols are defined by lex.yy.c, which is generated by flex. */

extern int yylex(void);
extern char *yytext;

/* These are the names of the styles we shall apply to the various 
   token types. 
*/

char *token_names[]={"fluffy","comment","keyword","preprocess","identifier",
		"number","string","symbol","whitespace","bracket"};

/* These are simple string functions that not only verify whether the
   destination buffer is sufficiently large, but they realloc() the
   buffer if it is not.  Highly convenient.
*/

void str_checklength(char **dest, size_t length_needed, size_t *length) {
 if ((*length < length_needed) || (dest == NULL)){
   *dest = realloc(*dest, length_needed);
   *length = length_needed;
 }
}

strrcpy(char **dest, char *src, size_t *length) {
 str_checklength(dest,strlen(src)+1,length);
 strncpy(*dest,src,*length);
}

strrcat(char **dest, char *src, size_t *length) {
 str_checklength(dest,strlen(src)+strlen(*dest)+1,length);
 strncat(*dest,src,*length);
}

void strrcatchar(char **dest, char chr, size_t *length) {
 char bob[2];
 bob[0] = chr;
 bob[1] = 0;
 strrcat(dest,bob,length);
 return;
}
 
/* There are certain symbols (notably '>' and '<') that must be
   escaped in the HTML document; otherwise, the document will not
   display correctly.  sanitize() searches a string for these
   symbols and replaces them with escape sequences. */
   
void sanitize(char **string, size_t *length) {
 char *dup;
 int i = 0;
 dup = strdup(*string);
 **string = 0;
 while (dup[i]) {
  switch (dup[i]) {
   case '>': strrcat(string,"&gt;",length); break;
   case '<': strrcat(string,"&lt;",length); break;
   case '&': strrcat(string,"&amp;",length); break;
   default:  strrcatchar(string,dup[i],length); break;
  } 
  i++;
 }
 free(dup);
}

int main(int argc, char **argv) {
 int x;
 char *string = NULL;
 size_t length = 0;
 printf("<html><head><link rel=\"stylesheet\" href=\"prettyc.css\"></head><body><pre>\n");
 while (x = yylex()) {
  strrcpy(&string,yytext,&length); 
  sanitize(&string,&length);
  printf("<span class=\"%s\">%s</span>",token_names[x],string); 
 }
 printf("</pre></body></html>\n");
 free(string);
 return 0;
}

